function [K, gamma, Delta, Xi, s2_3, err] = solve_struct( Q, psi, case1 )

% This function recovers the structural parameters from the estimated
% parameters using the procedure described in the appendix.  It used for
% calculating impulse responses.
%
% NOTICE:
% -------
% Permission is hereby granted to use this code freely for academic
% purposes only, provided that the paper is duly cited as:
%
% Eisenstat, E., Chan, J. C. C. and R. W. Strachan (2015). "Stochastic Model
% Specification Search for Time-Varying Parameter VARs," Econometric
% Reviews, forthcoming.
%
% Any other use of this code, particularly for comercial purposes, is
% strictly prohibited without the prior written consent of the authors.
% Furthermore, this code comes without technical support of any kind.  It
% is expected to reproduce the results reported in the paper.  However,
% beware that the notation may not the match text.  Likewise, it is not
% gauranteed to work under different hardware and / or software settings,
% other than those used for its original design and implementation.
%
% Under no circumstances will the authors be held responsible for any use
% (or misuse) of this code in any way.  If you do not agree to the above,
% you do not have permission to use this code.

err = false;
Q11 = Q( 1:2, 1:2 );
q12 = Q( 1:2, 3 );
q3 = Q( 3, 3 );

s2_3 = ( psi' * Q11 * psi - 2 * psi' * q12 + q3 ) / ( psi' * q12 - q3 ) ^ 2;
t = psi' * ( q12 - Q11 * psi );
cgamma1 = ( 1 + sqrt( 1 - 4 * s2_3 * t ) ) / ( 2 * t ) * ( q12 - Q11 * psi )';
cgamma2 = ( 1 - sqrt( 1 - 4 * s2_3 * t ) ) / ( 2 * t ) * ( q12 - Q11 * psi )';

if case1
    [cDelta1, cXi1] = ldl( inv( Q11 - cgamma1' * cgamma1 / s2_3 ) );
    [cDelta2, cXi2] = ldl( inv( Q11 - cgamma2' * cgamma2 / s2_3 ) );  
else
    [cDelta1, cXi1] = udl( inv( Q11 - cgamma1' * cgamma1 / s2_3 ) );
    [cDelta2, cXi2] = udl( inv( Q11 - cgamma2' * cgamma2 / s2_3 ) );  
end

W1 = [inv( cDelta1 ) zeros( 2, 1 ); 0 0 1] * [eye( 2 ) psi; cgamma1 1];
S1 = [cXi1 zeros( 2, 1 ); 0 0 s2_3];
K1 = sqrt( S1 ) \ W1;
Z1 = K1' * K1 - Q;

W2 = [inv( cDelta2 ) zeros( 2, 1 ); 0 0 1] * [eye( 2 ) psi; cgamma2 1];
S2 = [cXi2 zeros( 2, 1 ); 0 0 s2_3];
K2 = sqrt( S2 ) \ W2;
Z2 = K2' * K2 - Q;

if xor( any( abs( Z1( : ) ) > 1e-9 ), any( abs( Z2( : ) ) > 1e-9 ) )
    if all( abs( Z1( : ) ) <= 1e-9 )
        gamma = cgamma1;
        Delta = cDelta1;
        Xi = cXi1;
        K = K1;
    else
        gamma = cgamma2;
        Delta = cDelta2;
        Xi = cXi2;
        K = K2;
    end
else
    err = true;
    warning( 'No solution or multiple solutions found.' );
end

% last check
if any( ~isreal( K( : ) ) )
    err = true;
    warning( 'Some structural parameters are not real.' );
end

end

function [U, D] = udl( A )

n = size( A, 1 );
R = zeros( n, n );
R( n:( n - 1 ):( n * n - 1 ) ) = 1;
[L, D] = ldl( R * A * R );
U = R * L * R;
D = R * D * R;

end
