function y = tnr( mu, sig, lb, ub )
%%
% Univariate truncated normal sampling
% based on Robert (1995).  Uses the inverse
% CDF method if the closest bound is within
% certain number of standard deviations away
% from the mean; otherwise, the Robert (1995)
% rejection sampling techniques.
%
%     mu  - nx1 vector representing location params
%     sig - nx1 vector representing scale params
%     lb  - nx1 vector representing lower bounds
%     ub  - nx1 vector representing upper bounds
%
% All input vectors must be same-length column
% vectors.  Returns 'n' draws of independent
% truncated normal variables, each corresponding to
% the respective parameter inputs.
%
% NOTICE:
% -------
% Permission is hereby granted to use this code freely for academic
% purposes only, provided that the paper is duly cited as:
%
% Eisenstat, E., Chan, J. C. C. and R. W. Strachan (2015). "Stochastic Model
% Specification Search for Time-Varying Parameter VARs," Econometric
% Reviews, forthcoming.
%
% Any other use of this code, particularly for comercial purposes, is
% strictly prohibited without the prior written consent of the authors.
% Furthermore, this code comes without technical support of any kind.  It
% is expected to reproduce the results reported in the paper.  However,
% beware that the notation may not the match text.  Likewise, it is not
% gauranteed to work under different hardware and / or software settings,
% other than those used for its original design and implementation.
%
% Under no circumstances will the authors be held responsible for any use
% (or misuse) of this code in any way.  If you do not agree to the above,
% you do not have permission to use this code.

utol = 7;
ltol = -7;

n = length( mu );
a = ( lb - mu ) ./ sig;
b = ( ub - mu ) ./ sig;

idx_icdf = find( a < utol & b > ltol );
idx_ar = find( a >= utol | b <= ltol );
idx_arl = find( a( idx_ar ) > 0 );
idx_arr = find( b( idx_ar ) < 0 );

dbd_l = 2 * sqrt( exp( 1 ) ) ./ ( a + sqrt( a .^ 2 + 4 ) ) .* exp( ( a .^ 2 - a .* sqrt( a .^ 2 + 4 ) ) / 4 );
dbd_r = 2 * sqrt( exp( 1 ) ) ./ ( -b + sqrt( b .^ 2 + 4 ) ) .* exp( ( b .^ 2 + b .* sqrt( b .^ 2 + 4 ) ) / 4 );

idx_ar1l = find( b( idx_ar( idx_arl ) ) - a( idx_ar( idx_arl ) ) > dbd_l( idx_ar( idx_arl ) ) );
idx_ar2l = find( b( idx_ar( idx_arl ) ) - a( idx_ar( idx_arl ) ) <= dbd_l( idx_ar( idx_arl ) ) );
idx_ar1r = find( b( idx_ar( idx_arr ) ) - a( idx_ar( idx_arr ) ) > dbd_r( idx_ar( idx_arr ) ) );
idx_ar2r = find( b( idx_ar( idx_arr ) ) - a( idx_ar( idx_arr ) ) <= dbd_r( idx_ar( idx_arr ) ) );
idx_ar2 = [idx_arl( idx_ar2l ); idx_arr( idx_ar2r )];

x = repmat( NaN, n, 1 );

if ~isempty( idx_icdf )
    % inverse cdf method
    x( idx_icdf ) = tnr_icdf( a( idx_icdf ), b( idx_icdf ), length( idx_icdf ) );
end

if ~isempty( idx_ar1l )
    % left truncated
    x( idx_ar( idx_arl( idx_ar1l ) ) ) = tnr_ar1( a( idx_ar( idx_arl( idx_ar1l ) ) ), b( idx_ar( idx_arl( idx_ar1l ) ) ), length( idx_ar1l ) );
end

if ~isempty( idx_ar1r )
    % right truncated
    x( idx_ar( idx_arr( idx_ar1r ) ) ) = -tnr_ar1( -b( idx_ar( idx_arr( idx_ar1r ) ) ), -a( idx_ar( idx_arr( idx_ar1r ) ) ), length( idx_ar1r ) );
end

if ~isempty( idx_ar2 )
    % two-sided
    x( idx_ar( idx_ar2 ) ) = tnr_ar2( a( idx_ar( idx_ar2 ) ), b( idx_ar( idx_ar2 ) ), length( idx_ar( idx_ar2 ) ) );
end

y = mu + sig .* x;

end

function z = tnr_icdf( a, b, n )

p1 = normcdf( a );
p2 = normcdf( b );

z = norminv( p1 + rand( n, 1 ) .* ( p2 - p1 ) );

end

function z = tnr_ar1( a, b, n )

alpha = .5 * ( a + sqrt( a .^ 2 + 4 ) );

z = repmat( NaN, n, 1 );
idx = 1:n;

while ~isempty( idx )
    z_new = a( idx ) + exprnd( 1 ./ alpha( idx ) );
    r = exp( - ( z_new - alpha( idx ) ) .^ 2 / 2 );
    u = rand( length( idx ), 1 );
    z( idx( u <= r & z_new <= b( idx ) ) ) = z_new( u <= r & z_new <= b( idx ) );
    idx = idx( u > r | z_new > b( idx ) );
end

end

function z = tnr_ar2( a, b, n )
z = repmat( NaN, n, 1 );
idx = 1:n;

while ~isempty( idx )
    z_new = unifrnd( a( idx ), b( idx ) );

    c = a( idx ) .* ( a( idx ) > 0 ) + b( idx ) .* ( b( idx ) < 0 );
    r = exp( ( c .^ 2 - z_new .^ 2 ) / 2 );

    u = rand( length( idx ), 1 );
    z( idx( u <= r ) ) = z_new( u <= r );
    idx = idx( u > r );
end

end
